﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Diagnostics;

namespace RobobinApp.Networking
{
    public class WifiManager : IWifiService
    {
        private UdpClient _udpClient;
        private const int BroadcastPort = 5005;
        private bool _isConnected = false;
        private CancellationTokenSource _cancellationTokenSource;
        private TcpClient _tcpClient;

        // Event to notify the UI or other parts of the app of specific messages
        public event Action<string> OnMessageReceived;

        public WifiManager()
        {
            _udpClient = new UdpClient(BroadcastPort);
            _cancellationTokenSource = new CancellationTokenSource();
        }

        public async Task StartListening()
        {
            while (true) // Continuous listening loop with reconnection attempts
            {
                if (_isConnected)
                {
                    Debug.WriteLine("Already connected. Stopping further listening.");
                    break;
                }

                try
                {
                    Debug.WriteLine("Waiting for broadcast...");
                    var result = await _udpClient.ReceiveAsync();
                    string message = Encoding.ASCII.GetString(result.Buffer);

                    if (message == "ROBOBIN_PRESENT")
                    {
                        Debug.WriteLine("Detected Robobin presence from: " + result.RemoteEndPoint);
                        SendConnectMessage(result.RemoteEndPoint.Address.ToString());
                    }
                }
                catch (ObjectDisposedException)
                {
                    Debug.WriteLine("UDP client has been closed.");
                    break;
                }
                catch (Exception ex)
                {
                    Debug.WriteLine($"Error in UDP listening: {ex.Message}");
                }

                // Retry delay if not connected
                if (!_isConnected)
                {
                    await Task.Delay(3000); // Wait 3 seconds before retrying
                }
            }
        }

        public void SendConnectMessage(string ipAddress)
        {
            if (_isConnected)
            {
                Debug.WriteLine("Already connected. No need to send another connect message.");
                return;
            }

            var endPoint = new IPEndPoint(IPAddress.Parse(ipAddress), 5006);
            byte[] message = Encoding.ASCII.GetBytes("CONNECT");
            _udpClient.Send(message, message.Length, endPoint);
            Debug.WriteLine($"Sent connect message to: {ipAddress}");

            Task.Run(() => ConnectToTcpServer(endPoint));
        }

        public async Task SendMessageAsync(string message)
        {
            if (!_isConnected || _tcpClient == null)
            {
                Debug.WriteLine("Not connected. Cannot send message.");
                return;
            }

            try
            {
                NetworkStream stream = _tcpClient.GetStream();
                byte[] data = Encoding.ASCII.GetBytes(message);
                await stream.WriteAsync(data, 0, data.Length);
                Debug.WriteLine($"Sent message: {message}");
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"Failed to send message: {ex.Message}");
                _isConnected = false; // Reset connection status to retry
                await StartListening(); // Restart listening for reconnection
            }
        }

        public async Task SendPingMessage()
        {
            await SendMessageAsync("PING");
        }

        private async Task ConnectToTcpServer(IPEndPoint endPoint)
        {
            _tcpClient = new TcpClient();

            try
            {
                await _tcpClient.ConnectAsync(endPoint.Address, endPoint.Port);
                Debug.WriteLine("Connected to Robobin via TCP.");
                _isConnected = true;

 
                Task.Run(() => ReceiveMessages());

  
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"TCP connection failed: {ex.Message}");
                _isConnected = false; // Reset connection status
                await StartListening(); // Retry listening for presence broadcast
            }

            // If TCP connection is lost, attempt to reconnect
            _cancellationTokenSource.Cancel(); // Stop the current listening task
        }

        private async Task ReceiveMessages()
        {
            if (_tcpClient == null)
                return;

            NetworkStream stream = _tcpClient.GetStream();
            byte[] buffer = new byte[1024];

            try
            {
                while (_isConnected)
                {
                    int byteCount = await stream.ReadAsync(buffer, 0, buffer.Length);
                    if (byteCount <= 0)
                    {
                        Debug.WriteLine("Disconnected from server.");
                        _isConnected = false;
                        break;
                    }

                    string receivedMessage = Encoding.ASCII.GetString(buffer, 0, byteCount);
                    Debug.WriteLine($"Received message: {receivedMessage}");

                    // Trigger alert or handle specific messages
                    HandleReceivedMessage(receivedMessage);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"Error receiving message: {ex.Message}");
                _isConnected = false;
                await StartListening(); // Restart listening if disconnected
            }
        }

        private void HandleReceivedMessage(string message)
        {
            if (message == "PONG")
            {
                OnMessageReceived?.Invoke("Received PONG from Robobin");
                Debug.WriteLine("PONG received, alert triggered.");
            }
            else
            {
                OnMessageReceived?.Invoke($"RM: {message}");
            }
        }
    }
}
